__author__ = "hugo.inzirillo"


class TemplateHeader(object):
    def __init__(self):
        pass

    def __dict__(self):
        pass

    def __str__(self):
        pass

    def __repr__(self):
        pass



